/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

/**
 * originally from https://github.com/darkreader/darkreader/blob/c24ed5ac4ae10d47dbc348354220ecf5d2113bfd/src/utils/color.ts
 *
 * MIT License
 *
 * Copyright (c) 2019 Alexander Shutau
 * Copyright (small modifications) (c) 2020 Philipp Mildenberger
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var knownColors = new Map(Object.entries({
    aliceblue: 0xf0f8ff,
    antiquewhite: 0xfaebd7,
    aqua: 0x00ffff,
    aquamarine: 0x7fffd4,
    azure: 0xf0ffff,
    beige: 0xf5f5dc,
    bisque: 0xffe4c4,
    black: 0x000000,
    blanchedalmond: 0xffebcd,
    blue: 0x0000ff,
    blueviolet: 0x8a2be2,
    brown: 0xa52a2a,
    burlywood: 0xdeb887,
    cadetblue: 0x5f9ea0,
    chartreuse: 0x7fff00,
    chocolate: 0xd2691e,
    coral: 0xff7f50,
    cornflowerblue: 0x6495ed,
    cornsilk: 0xfff8dc,
    crimson: 0xdc143c,
    cyan: 0x00ffff,
    darkblue: 0x00008b,
    darkcyan: 0x008b8b,
    darkgoldenrod: 0xb8860b,
    darkgray: 0xa9a9a9,
    darkgrey: 0xa9a9a9,
    darkgreen: 0x006400,
    darkkhaki: 0xbdb76b,
    darkmagenta: 0x8b008b,
    darkolivegreen: 0x556b2f,
    darkorange: 0xff8c00,
    darkorchid: 0x9932cc,
    darkred: 0x8b0000,
    darksalmon: 0xe9967a,
    darkseagreen: 0x8fbc8f,
    darkslateblue: 0x483d8b,
    darkslategray: 0x2f4f4f,
    darkslategrey: 0x2f4f4f,
    darkturquoise: 0x00ced1,
    darkviolet: 0x9400d3,
    deeppink: 0xff1493,
    deepskyblue: 0x00bfff,
    dimgray: 0x696969,
    dimgrey: 0x696969,
    dodgerblue: 0x1e90ff,
    firebrick: 0xb22222,
    floralwhite: 0xfffaf0,
    forestgreen: 0x228b22,
    fuchsia: 0xff00ff,
    gainsboro: 0xdcdcdc,
    ghostwhite: 0xf8f8ff,
    gold: 0xffd700,
    goldenrod: 0xdaa520,
    gray: 0x808080,
    grey: 0x808080,
    green: 0x008000,
    greenyellow: 0xadff2f,
    honeydew: 0xf0fff0,
    hotpink: 0xff69b4,
    indianred: 0xcd5c5c,
    indigo: 0x4b0082,
    ivory: 0xfffff0,
    khaki: 0xf0e68c,
    lavender: 0xe6e6fa,
    lavenderblush: 0xfff0f5,
    lawngreen: 0x7cfc00,
    lemonchiffon: 0xfffacd,
    lightblue: 0xadd8e6,
    lightcoral: 0xf08080,
    lightcyan: 0xe0ffff,
    lightgoldenrodyellow: 0xfafad2,
    lightgray: 0xd3d3d3,
    lightgrey: 0xd3d3d3,
    lightgreen: 0x90ee90,
    lightpink: 0xffb6c1,
    lightsalmon: 0xffa07a,
    lightseagreen: 0x20b2aa,
    lightskyblue: 0x87cefa,
    lightslategray: 0x778899,
    lightslategrey: 0x778899,
    lightsteelblue: 0xb0c4de,
    lightyellow: 0xffffe0,
    lime: 0x00ff00,
    limegreen: 0x32cd32,
    linen: 0xfaf0e6,
    magenta: 0xff00ff,
    maroon: 0x800000,
    mediumaquamarine: 0x66cdaa,
    mediumblue: 0x0000cd,
    mediumorchid: 0xba55d3,
    mediumpurple: 0x9370db,
    mediumseagreen: 0x3cb371,
    mediumslateblue: 0x7b68ee,
    mediumspringgreen: 0x00fa9a,
    mediumturquoise: 0x48d1cc,
    mediumvioletred: 0xc71585,
    midnightblue: 0x191970,
    mintcream: 0xf5fffa,
    mistyrose: 0xffe4e1,
    moccasin: 0xffe4b5,
    navajowhite: 0xffdead,
    navy: 0x000080,
    oldlace: 0xfdf5e6,
    olive: 0x808000,
    olivedrab: 0x6b8e23,
    orange: 0xffa500,
    orangered: 0xff4500,
    orchid: 0xda70d6,
    palegoldenrod: 0xeee8aa,
    palegreen: 0x98fb98,
    paleturquoise: 0xafeeee,
    palevioletred: 0xdb7093,
    papayawhip: 0xffefd5,
    peachpuff: 0xffdab9,
    peru: 0xcd853f,
    pink: 0xffc0cb,
    plum: 0xdda0dd,
    powderblue: 0xb0e0e6,
    purple: 0x800080,
    rebeccapurple: 0x663399,
    red: 0xff0000,
    rosybrown: 0xbc8f8f,
    royalblue: 0x4169e1,
    saddlebrown: 0x8b4513,
    salmon: 0xfa8072,
    sandybrown: 0xf4a460,
    seagreen: 0x2e8b57,
    seashell: 0xfff5ee,
    sienna: 0xa0522d,
    silver: 0xc0c0c0,
    skyblue: 0x87ceeb,
    slateblue: 0x6a5acd,
    slategray: 0x708090,
    slategrey: 0x708090,
    snow: 0xfffafa,
    springgreen: 0x00ff7f,
    steelblue: 0x4682b4,
    tan: 0xd2b48c,
    teal: 0x008080,
    thistle: 0xd8bfd8,
    tomato: 0xff6347,
    turquoise: 0x40e0d0,
    violet: 0xee82ee,
    wheat: 0xf5deb3,
    white: 0xffffff,
    whitesmoke: 0xf5f5f5,
    yellow: 0xffff00,
    yellowgreen: 0x9acd32,
}));
var systemColors = new Map(Object.entries({
    ActiveBorder: 0x3b99fc,
    ActiveCaption: 0x000000,
    AppWorkspace: 0xaaaaaa,
    Background: 0x6363ce,
    ButtonFace: 0xffffff,
    ButtonHighlight: 0xe9e9e9,
    ButtonShadow: 0x9fa09f,
    ButtonText: 0x000000,
    CaptionText: 0x000000,
    GrayText: 0x7f7f7f,
    Highlight: 0xb2d7ff,
    HighlightText: 0x000000,
    InactiveBorder: 0xffffff,
    InactiveCaption: 0xffffff,
    InactiveCaptionText: 0x000000,
    InfoBackground: 0xfbfcc5,
    InfoText: 0x000000,
    Menu: 0xf6f6f6,
    MenuText: 0xffffff,
    Scrollbar: 0xaaaaaa,
    ThreeDDarkShadow: 0x000000,
    ThreeDFace: 0xc0c0c0,
    ThreeDHighlight: 0xffffff,
    ThreeDLightShadow: 0xffffff,
    ThreeDShadow: 0x000000,
    Window: 0xececec,
    WindowFrame: 0xaaaaaa,
    WindowText: 0x000000,
    '-webkit-focus-ring-color': 0xe59700,
}).map(function (_a) {
    var key = _a[0], value = _a[1];
    return [key.toLowerCase(), value];
}));
// https://en.wikipedia.org/wiki/HSL_and_HSV
function hslToRGB(_a) {
    var h = _a.h, s = _a.s, l = _a.l, _b = _a.a, a = _b === void 0 ? 1 : _b;
    if (s === 0) {
        var _c = [l, l, l].map(function (x) { return Math.round(x * 255); }), r_1 = _c[0], b_1 = _c[1], g_1 = _c[2];
        return { r: r_1, g: g_1, b: b_1, a: a };
    }
    var c = (1 - Math.abs(2 * l - 1)) * s;
    var x = c * (1 - Math.abs(((h / 60) % 2) - 1));
    var m = l - c / 2;
    var _d = (h < 60
        ? [c, x, 0]
        : h < 120
            ? [x, c, 0]
            : h < 180
                ? [0, c, x]
                : h < 240
                    ? [0, x, c]
                    : h < 300
                        ? [x, 0, c]
                        : [c, 0, x]).map(function (n) { return Math.round((n + m) * 255); }), r = _d[0], g = _d[1], b = _d[2];
    return { r: r, g: g, b: b, a: a };
}
// https://en.wikipedia.org/wiki/HSL_and_HSV
function rgbToHSL(_a) {
    var r255 = _a.r, g255 = _a.g, b255 = _a.b, _b = _a.a, a = _b === void 0 ? 1 : _b;
    var r = r255 / 255;
    var g = g255 / 255;
    var b = b255 / 255;
    var max = Math.max(r, g, b);
    var min = Math.min(r, g, b);
    var c = max - min;
    var l = (max + min) / 2;
    if (c === 0) {
        return { h: 0, s: 0, l: l, a: a };
    }
    var h = (max === r ? ((g - b) / c) % 6 : max === g ? (b - r) / c + 2 : (r - g) / c + 4) * 60;
    if (h < 0) {
        h += 360;
    }
    var s = c / (1 - Math.abs(2 * l - 1));
    return { h: h, s: s, l: l, a: a };
}
function toFixed(n, digits) {
    if (digits === void 0) { digits = 0; }
    var fixed = n.toFixed(digits);
    if (digits === 0) {
        return fixed;
    }
    var dot = fixed.indexOf('.');
    var zerosMatch = fixed.match(/0+$/);
    if (zerosMatch) {
        if (zerosMatch.index === dot + 1) {
            return fixed.substring(0, dot);
        }
        return fixed.substring(0, zerosMatch.index);
    }
    return fixed;
}
function rgbToString(rgb) {
    var r = rgb.r, g = rgb.g, b = rgb.b, a = rgb.a;
    if (a != null && a < 1) {
        return "rgba(" + toFixed(r) + ", " + toFixed(g) + ", " + toFixed(b) + ", " + toFixed(a, 2) + ")";
    }
    return "rgb(" + toFixed(r) + ", " + toFixed(g) + ", " + toFixed(b) + ")";
}
function rgbToHexString(_a) {
    var r = _a.r, g = _a.g, b = _a.b, a = _a.a;
    return "#" + (a != null && a < 1 ? [r, g, b, a * 255] : [r, g, b])
        .map(function (x) {
        return "" + (x < 16 ? '0' : '') + Math.round(x).toString(16);
    })
        .join('');
}
function hslToString(hsl) {
    var h = hsl.h, s = hsl.s, l = hsl.l, a = hsl.a;
    if (a != null && a < 1) {
        return "hsla(" + toFixed(h) + ", " + toFixed(s * 100) + "%, " + toFixed(l * 100) + "%, " + toFixed(a, 2) + ")";
    }
    return "hsl(" + toFixed(h) + ", " + toFixed(s * 100) + "%, " + toFixed(l * 100) + "%)";
}
function getNumbersFromString(str, splitter, range, units) {
    var raw = str.split(splitter).filter(function (x) { return x; });
    var unitsList = Object.entries(units);
    var numbers = raw
        .map(function (r) { return r.trim(); })
        .map(function (r, i) {
        var n;
        var unit = unitsList.find(function (_a) {
            var u = _a[0];
            return r.endsWith(u);
        });
        if (unit) {
            n = (parseFloat(r.substring(0, r.length - unit[0].length)) / unit[1]) * range[i];
        }
        else {
            n = parseFloat(r);
        }
        if (range[i] > 1) {
            return Math.round(n);
        }
        return n;
    });
    return numbers;
}
var rgbSplitter = /rgba?|\(|\)|\/|,|\s/gi;
var rgbRange = [255, 255, 255, 1];
var rgbUnits = { '%': 100 };
function parseRGB($rgb) {
    var _a = getNumbersFromString($rgb, rgbSplitter, rgbRange, rgbUnits), r = _a[0], g = _a[1], b = _a[2], _b = _a[3], a = _b === void 0 ? 1 : _b;
    return { r: r, g: g, b: b, a: a };
}
var hslSplitter = /hsla?|\(|\)|\/|,|\s/gi;
var hslRange = [360, 1, 1, 1];
var hslUnits = { '%': 100, deg: 360, rad: 2 * Math.PI, turn: 1 };
function parseHSL($hsl) {
    var _a = getNumbersFromString($hsl, hslSplitter, hslRange, hslUnits), h = _a[0], s = _a[1], l = _a[2], _b = _a[3], a = _b === void 0 ? 1 : _b;
    return hslToRGB({ h: h, s: s, l: l, a: a });
}
function parseHex($hex) {
    var h = $hex.substring(1);
    var l = h.length;
    if (l === 3 || l === 4) {
        var _a = [0, 1, 2].map(function (i) { return parseInt("" + h[i] + h[i], 16); }), r = _a[0], g = _a[1], b = _a[2];
        var a = h.length === 3 ? 1 : parseInt("" + h[3] + h[3], 16) / 255;
        return { r: r, g: g, b: b, a: a };
    }
    if (l === 6 || l === 8) {
        var _b = [0, 2, 4].map(function (i) { return parseInt(h.substring(i, i + 2), 16); }), r = _b[0], g = _b[1], b = _b[2];
        var a = h.length === 6 ? 1 : parseInt(h.substring(6, 8), 16) / 255;
        return { r: r, g: g, b: b, a: a };
    }
    throw new Error("Unable to parse " + $hex);
}
function getColorByName($color) {
    var n = knownColors.get($color) || 0;
    return {
        r: (n >> 16) & 255,
        g: (n >> 8) & 255,
        b: (n >> 0) & 255,
        a: 1,
    };
}
function getSystemColor($color) {
    var n = systemColors.get($color) || 0;
    return {
        r: (n >> 16) & 255,
        g: (n >> 8) & 255,
        b: (n >> 0) & 255,
        a: 1,
    };
}
var rgbMatch = /^rgba?\([^()]+\)$/;
var hslMatch = /^hsla?\([^()]+\)$/;
var hexMatch = /^#[0-9a-f]+$/i;
function parse($color) {
    var c = $color.trim().toLowerCase();
    if (c.match(rgbMatch)) {
        return parseRGB(c);
    }
    if (c.match(hslMatch)) {
        return parseHSL(c);
    }
    if (c.match(hexMatch)) {
        return parseHex(c);
    }
    if (knownColors.has(c)) {
        return getColorByName(c);
    }
    if (systemColors.has(c)) {
        return getSystemColor(c);
    }
    if ($color === 'transparent') {
        return { r: 0, g: 0, b: 0, a: 0 };
    }
    throw new Error("Unable to parse " + $color);
}

function colorToRGBA(color) {
    if (typeof color === 'string') {
        return parse(color);
    }
    if (Object.prototype.hasOwnProperty.call(color, 'r')) {
        return __assign({}, color);
    }
    return hslToRGB(color);
}
function convert(color, outputType, convertFunction) {
    if (convertFunction === void 0) { convertFunction = function (c) { return c; }; }
    var colorResult = convertFunction(colorToRGBA(color));
    switch (outputType) {
        case 'rgb':
            return colorResult;
        case 'hsl':
            return rgbToHSL(colorResult);
        case 'css-rgb':
            return rgbToString(colorResult);
        case 'css-hex':
            return rgbToHexString(colorResult);
        case 'css-hsl':
            return hslToString(rgbToHSL(colorResult));
        default:
            throw new Error("Unsupported outputType '" + outputType + "'");
    }
}
// -------------------------------------------------------- conversion functions
function invert(c) {
    return {
        r: 255 - c.r,
        g: 255 - c.g,
        b: 255 - c.b,
        a: c.a === undefined ? 1.0 : c.a,
    };
}
function getLuminance(c) {
    var l = function (v) {
        var n = v / 255;
        return n <= 0.03928 ? n / 12.92 : Math.pow(((n + 0.055) / 1.055), 2.4);
    };
    return 0.2126 * l(c.r) + 0.7152 * l(c.g) + 0.0722 * l(c.b);
}
function invertDarkLight(_a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.threshold, threshold = _c === void 0 ? Math.sqrt(1.05 * 0.05) - 0.05 : _c, _d = _b.dark, dark = _d === void 0 ? 'black' : _d, _e = _b.light, light = _e === void 0 ? 'white' : _e, _f = _b.useInputAlpha, useInputAlpha = _f === void 0 ? false : _f;
    var _dark = colorToRGBA(dark);
    var _light = colorToRGBA(light);
    return function (c) {
        var result = getLuminance(c) > threshold ? _dark : _light;
        return useInputAlpha ? __assign(__assign({}, result), { a: c.a === undefined ? 1.0 : c.a }) : result;
    };
}
function gammaCorrection(_a) {
    var _b = (_a === void 0 ? {} : _a).factor, factor = _b === void 0 ? 2.2 : _b;
    var f = function (v) { return 255.0 * Math.pow((v / 255.0), (1.0 / factor)); };
    return function (c) {
        return {
            r: f(c.r),
            g: f(c.g),
            b: f(c.b),
            a: c.a === undefined ? 1.0 : c.a,
        };
    };
}
function contrastBrightness(_a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.contrast, contrast = _c === void 0 ? 1.0 : _c, _d = _b.brightness, brightness = _d === void 0 ? 0.0 : _d;
    var f = function (v) { return Math.min(255, Math.max(0, contrast * (v - 128) + 128 + brightness)); };
    return function (c) {
        return {
            r: f(c.r),
            g: f(c.g),
            b: f(c.b),
            a: c.a === undefined ? 1.0 : c.a,
        };
    };
}

export { colorToRGBA, contrastBrightness, convert, gammaCorrection, hslToRGB, hslToString, invert, invertDarkLight, parse, rgbToHSL, rgbToHexString, rgbToString };
